==> contentsquare-2.0.0.tar.gz: test/helpers/matchers/fixtures/matcher.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:flutter_test/flutter_test.dart';

import '../../framework.dart';

part 'json_encoder.dart';

/// We define [_flutterVersion] as a global variable to avoid recomputing
/// since it cannot change within a single run of flutter test.
late String _flutterVersion = getFlutterVersion();

const _prettyJsonEncoder = JsonEncoder.withIndent('  ');

/// Creates a matcher that matches a String against a fixture.
///
/// [description] must be the unique id of the fixture.
///
/// [isVersionSpecific] must be true if the fixture depends on the Flutter
/// version.
///
///
/// ## Update the fixtures
///
/// Use `--update-goldens` to update the fixtures.
///
/// Note that a fixture is specific to a Flutter version, therefore only the
/// fixtures for the current Flutter version will be updated.
///
///
/// ## Where are the fixtures stored?
///
/// Fixtures of a test are stored in:
///   test/fixtures/<flutter-version>/<path-to-test-file>/<test-file>_fixtures.json`
///
/// Inside the file, the fixtures are stored in a map with the following
/// structure:
///  {
///     '<fixture-name-1>': '<fixture-content-1>',
///     '<fixture-name-2>': '<fixture-content-2>',
///     ...
///  }
///
/// Example:
///
/// test/my_test.dart:
/// ```dart
/// test(
///   'Description of my test',
///   () async {
///     const randomTestFixture = 'MyFixtureToTest';
///
///     expect(
///       randomTestFixture,
///       matchesFixture('FixtureToTest'),
///     );
///   },
/// );
/// ```
///
/// JSON output of --update-goldens (on Flutter 2.10.5):
/// test/fixtures/2_10_5/my_test_fixtures.json:
/// ```json
/// {
///   "FixtureToTest": "MyFixtureToTest",
/// }
/// ```
_VersionedFixtureMatcher matchesFixture(
  String description, {
  bool isVersionSpecific = true,
}) =>
    _VersionedFixtureMatcher(
      fixtureId: description,
      isVersionSpecific: isVersionSpecific,
    );

class _VersionedFixtureMatcher extends Matcher {
  _VersionedFixtureMatcher({
    required this.fixtureId,
    required this.isVersionSpecific,
  });

  /// Fixtures of the current test which **are** version specific.
  /// It is null while the fixtures are not loaded.
  ///
  /// This is used to avoid reloading the fixtures for each test of a test file.
  static Map<String, Object?>? _fileVersionSpecificFixtures;

  /// Fixtures of the current test which are **not** version specific.
  /// It is null while the fixtures are not loaded.
  ///
  /// This is used to avoid reloading the fixtures for each test of a test file.
  static Map<String, Object?>? _fileVersionsSharedFixtures;

  /// True if the fixtures depend on the Flutter version.
  final bool isVersionSpecific;

  /// A unique identifier for the fixture.
  final String fixtureId;

  late final Matcher _matcher;

  /// Returns the file containing the fixtures of the current test and the
  /// current Flutter version.
  File _getFixturesFile() {
    final fixtureFilePath = _getVersionedFixtureFilePath(
      fixtureId: fixtureId,
      flutterVersion: isVersionSpecific ? _flutterVersion : 'every',
    );

    return File(fixtureFilePath.path);
  }

  /// Loads the fixtures from the file associated with the current test and the
  /// current Flutter version.
  Map<String, Object?> _loadFixtures() {
    final fileVersionSpecificFixtures = _fileVersionSpecificFixtures;
    final fileVersionsSharedFixtures = _fileVersionsSharedFixtures;
    if (isVersionSpecific && fileVersionSpecificFixtures != null) {
      return fileVersionSpecificFixtures;
    }
    if (!isVersionSpecific && fileVersionsSharedFixtures != null) {
      return fileVersionsSharedFixtures;
    }

    final fixtureFile = _getFixturesFile();

    if (!fixtureFile.existsSync()) {
      throw _FixtureFileDoesNotExistsException(file: fixtureFile.path);
    }

    final encodedFixturesJson = fixtureFile.readAsStringSync();

    final fixtures = json.decode(encodedFixturesJson) as Map<String, Object?>;

    if (isVersionSpecific) {
      _fileVersionSpecificFixtures = fixtures;
    } else {
      _fileVersionsSharedFixtures = fixtures;
    }

    return fixtures;
  }

  /// Returns the fixtures of id [fixtureId] associated with the current Flutter
  /// version.
  ///
  /// Throws if the file does not exists.
  Object? _loadFixture() {
    // Get the fixtures for every tests of the file.
    //
    // If [_VersionedFixtureMatcher._fileFixtures] is null, assign it the value
    // from the files and store its value in [_fileFixtures].
    // If [_VersionedFixtureMatcher._fileFixtures] is NOT null, store its value
    // in [_fileFixtures].
    final fileFixtures = _loadFixtures();

    final fixture = fileFixtures[fixtureId];

    if (fixture == null) {
      throw Exception(
        'Fixture does not exist for test: $fixtureId, '
        'flutterVersion: $_flutterVersion. '
        'Use --update-goldens to generate it. '
        'All fixtures: $fileFixtures',
      );
    }

    return fixture;
  }

  void _saveFixture({required Object? fixture}) {
    // This map contains the fixtures for every tests of the file.
    final fixturesJson = () {
      try {
        return _loadFixtures();
      } on _FixtureFileDoesNotExistsException {
        return <String, Object?>{};
      }
    }();

    fixturesJson[fixtureId] = fixture;

    final fixturesString = _prettyJsonEncoder.convert(fixturesJson);

    final fixtureFile = _getFixturesFile();

    if (!fixtureFile.existsSync()) {
      fixtureFile.createSync(recursive: true);
    }

    fixtureFile.writeAsStringSync(fixturesString);
  }

  static String _getMainFilePath() {
    // File of the [main] function currently running.
    final mainFile = Uri.parse(
      RegExp(r'#.*main.*\(file:\/\/(.*):.*:.*\)')
          .firstMatch(StackTrace.current.toString())!
          .group(1)!,
    ).toFilePath();
    return mainFile;
  }

  /// Returns the path of the fixture file for the current test.
  ///
  /// This mirrors the path to the test with a fixture folder inside /test
  ///
  /// Example for flutterVersion=2.10.5:
  /// - Test file: test/src/modules/my_module/my_test.dart
  /// - Fixtures file: test/fixtures/2_10_5/src/modules/my_module/my_test_fixtures.dart
  static Uri _getVersionedFixtureFilePath({
    required String fixtureId,
    required String flutterVersion,
  }) {
    // 2.10.5 -> 2_10_5
    // This is needed because file names cannot contain '.'.
    final lowDashedFlutterVersion = flutterVersion.replaceAll('.', '_');

    final mainFilePath = _getMainFilePath();

    // Path to the /test folder.
    const testFolderDelimiter = '/test/';
    final testFilePath = mainFilePath.substring(
      0,
      mainFilePath.lastIndexOf(testFolderDelimiter) +
          testFolderDelimiter.length -
          1,
    );

    // Path from the /test folder to the current file
    final mainFilePathFromTest = mainFilePath.substring(testFilePath.length);

    final mainFilePathFromTestWithoutExtension =
        mainFilePathFromTest.substring(1, mainFilePathFromTest.indexOf('.'));

    final fixturesFolder =
        '$testFilePath/fixtures/$lowDashedFlutterVersion/${mainFilePathFromTestWithoutExtension}_fixtures.json';

    return Uri.file(fixturesFolder);
  }

  @override
  bool matches(dynamic item, Map matchState) {
    item = const _FixtureEncoder().convert(item);

    try {
      if (autoUpdateGoldenFiles) {
        _saveFixture(fixture: item);

        return true;
      }

      _matcher = equals(_loadFixture());

      return _matcher.matches(item, matchState);
    } catch (e, stackTrace) {
      // The [matches] function should not throw (as this would interrupt the
      // other tests).
      //
      // Instead, when an exception occurs, we return false and return a
      // description of the error in [describeMismatch] with the information
      // stored in [matchState].
      matchState[_internalErrorKey] = e;
      matchState[_internalStackTraceKey] = stackTrace;
      return false;
    }
  }

  static const _internalErrorKey = 'internal_error';
  static const _internalStackTraceKey = 'internal_error_stackTrace';

  @override
  Description describe(Description description) =>
      description.addDescriptionOf(description);

  @override
  Description describeMismatch(
    item,
    Description description,
    Map matchState,
    bool verbose,
  ) {
    // If an error was thrown in [matches] we print it from here
    if (matchState.containsKey(_internalErrorKey)) {
      // Add the error and stack trace to the description.
      final error = matchState[_internalErrorKey];
      final stackTrace = matchState[_internalStackTraceKey];
      return description
          .add('Failed:\n')
          .add('$error\n\n')
          .add('🔍 Detailed Stack trace:\n$stackTrace\n');
    }

    return _matcher.describeMismatch(
      item,
      description,
      matchState,
      verbose,
    );
  }
}

class _FixtureFileDoesNotExistsException implements Exception {
  final String file;

  _FixtureFileDoesNotExistsException({required this.file});

  @override
  String toString() => 'Fixture File does not exist: $file';
}

==> contentsquare-2.3.0.tar.gz: test/helpers/matchers/fixtures/matcher.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:flutter_test/flutter_test.dart';

import '../../framework.dart';

part 'json_encoder.dart';

/// We define [_flutterVersion] as a global variable to avoid recomputing
/// since it cannot change within a single run of flutter test.
String _flutterVersion = getFlutterVersion();

const _prettyJsonEncoder = JsonEncoder.withIndent('  ');

/// Creates a matcher that matches a String against a fixture.
///
/// [description] must be the unique id of the fixture.
///
/// [isVersionSpecific] must be true if the fixture depends on the Flutter
/// version.
///
///
/// ## Update the fixtures
///
/// Use `--update-goldens` to update the fixtures.
///
/// Note that a fixture is specific to a Flutter version, therefore only the
/// fixtures for the current Flutter version will be updated.
///
///
/// ## Where are the fixtures stored?
///
/// Fixtures of a test are stored in:
///   test/fixtures/<flutter-version>/<path-to-test-file>/<test-file>_fixtures.json`
///
/// Inside the file, the fixtures are stored in a map with the following
/// structure:
///  {
///     '<fixture-name-1>': '<fixture-content-1>',
///     '<fixture-name-2>': '<fixture-content-2>',
///     ...
///  }
///
/// Example:
///
/// test/my_test.dart:
/// ```dart
/// test(
///   'Description of my test',
///   () async {
///     const randomTestFixture = 'MyFixtureToTest';
///
///     expect(
///       randomTestFixture,
///       matchesFixture('FixtureToTest'),
///     );
///   },
/// );
/// ```
///
/// JSON output of --update-goldens (on Flutter 2.10.5):
/// test/fixtures/2_10_5/my_test_fixtures.json:
/// ```json
/// {
///   "FixtureToTest": "MyFixtureToTest",
/// }
/// ```
Matcher matchesFixture(
  String description, {
  bool isVersionSpecific = true,
}) =>
    _VersionedFixtureMatcher(
      fixtureId: description,
      isVersionSpecific: isVersionSpecific,
    );

class _VersionedFixtureMatcher extends Matcher {
  _VersionedFixtureMatcher({
    required this.fixtureId,
    required this.isVersionSpecific,
  });

  /// Fixtures of the current test which **are** version specific.
  /// It is null while the fixtures are not loaded.
  ///
  /// This is used to avoid reloading the fixtures for each test of a test file.
  static Map<String, Object?>? _fileVersionSpecificFixtures;

  /// Fixtures of the current test which are **not** version specific.
  /// It is null while the fixtures are not loaded.
  ///
  /// This is used to avoid reloading the fixtures for each test of a test file.
  static Map<String, Object?>? _fileVersionsSharedFixtures;

  /// True if the fixtures depend on the Flutter version.
  final bool isVersionSpecific;

  /// A unique identifier for the fixture.
  final String fixtureId;

  late final Matcher _matcher;

  /// Returns the file containing the fixtures of the current test and the
  /// current Flutter version.
  File _getFixturesFile() {
    final fixtureFilePath = _getVersionedFixtureFilePath(
      fixtureId: fixtureId,
      flutterVersion: isVersionSpecific ? _flutterVersion : 'every',
    );

    return File(fixtureFilePath.path);
  }

  /// Loads the fixtures from the file associated with the current test and the
  /// current Flutter version.
  Map<String, Object?> _loadFixtures() {
    final fileVersionSpecificFixtures = _fileVersionSpecificFixtures;
    final fileVersionsSharedFixtures = _fileVersionsSharedFixtures;
    if (isVersionSpecific && fileVersionSpecificFixtures != null) {
      return fileVersionSpecificFixtures;
    }
    if (!isVersionSpecific && fileVersionsSharedFixtures != null) {
      return fileVersionsSharedFixtures;
    }

    final fixtureFile = _getFixturesFile();

    if (!fixtureFile.existsSync()) {
      throw _FixtureFileDoesNotExistsException(file: fixtureFile.path);
    }

    final encodedFixturesJson = fixtureFile.readAsStringSync();

    final fixtures = json.decode(encodedFixturesJson) as Map<String, Object?>;

    if (isVersionSpecific) {
      _fileVersionSpecificFixtures = fixtures;
    } else {
      _fileVersionsSharedFixtures = fixtures;
    }

    return fixtures;
  }

  /// Returns the fixtures of id [fixtureId] associated with the current Flutter
  /// version.
  ///
  /// Throws if the file does not exists.
  Object? _loadFixture() {
    // Get the fixtures for every tests of the file.
    //
    // If [_VersionedFixtureMatcher._fileFixtures] is null, assign it the value
    // from the files and store its value in [_fileFixtures].
    // If [_VersionedFixtureMatcher._fileFixtures] is NOT null, store its value
    // in [_fileFixtures].
    final fileFixtures = _loadFixtures();

    final fixture = fileFixtures[fixtureId];

    if (fixture == null) {
      throw Exception(
        'Fixture does not exist for test: $fixtureId, '
        'flutterVersion: $_flutterVersion. '
        'Use --update-goldens to generate it. '
        'All fixtures: $fileFixtures',
      );
    }

    return fixture;
  }

  void _saveFixture({required Object? fixture}) {
    // This map contains the fixtures for every tests of the file.
    final fixturesJson = () {
      try {
        return _loadFixtures();
      } on _FixtureFileDoesNotExistsException {
        return <String, Object?>{};
      }
    }();

    fixturesJson[fixtureId] = fixture;

    final fixturesString = _prettyJsonEncoder.convert(fixturesJson);

    final fixtureFile = _getFixturesFile();

    if (!fixtureFile.existsSync()) {
      fixtureFile.createSync(recursive: true);
    }

    fixtureFile.writeAsStringSync(fixturesString);
  }

  static String _getMainFilePath() {
    // File of the [main] function currently running.
    final mainFile = Uri.parse(
      RegExp(r'#.*main.*\(file:\/\/(.*):.*:.*\)')
          .firstMatch(StackTrace.current.toString())!
          .group(1)!,
    ).toFilePath();
    return mainFile;
  }

  /// Returns the path of the fixture file for the current test.
  ///
  /// This mirrors the path to the test with a fixture folder inside /test
  ///
  /// Example for flutterVersion=2.10.5:
  /// - Test file: test/src/modules/my_module/my_test.dart
  /// - Fixtures file: test/fixtures/2_10_5/src/modules/my_module/my_test_fixtures.dart
  static Uri _getVersionedFixtureFilePath({
    required String fixtureId,
    required String flutterVersion,
  }) {
    // 2.10.5 -> 2_10_5
    // This is needed because file names cannot contain '.'.
    final lowDashedFlutterVersion = flutterVersion.replaceAll('.', '_');

    final mainFilePath = _getMainFilePath();

    // Path to the /test folder.
    const testFolderDelimiter = '/test/';
    final testFilePath = mainFilePath.substring(
      0,
      mainFilePath.lastIndexOf(testFolderDelimiter) +
          testFolderDelimiter.length -
          1,
    );

    // Path from the /test folder to the current file
    final mainFilePathFromTest = mainFilePath.substring(testFilePath.length);

    final mainFilePathFromTestWithoutExtension =
        mainFilePathFromTest.substring(1, mainFilePathFromTest.indexOf('.'));

    final fixturesFolder =
        '$testFilePath/fixtures/$lowDashedFlutterVersion/${mainFilePathFromTestWithoutExtension}_fixtures.json';

    return Uri.file(fixturesFolder);
  }

  @override
  bool matches(dynamic item, Map matchState) {
    item = const _FixtureEncoder().convert(item);

    try {
      if (autoUpdateGoldenFiles) {
        _saveFixture(fixture: item);

        return true;
      }

      _matcher = equals(_loadFixture());

      return _matcher.matches(item, matchState);
    } catch (e, stackTrace) {
      // The [matches] function should not throw (as this would interrupt the
      // other tests).
      //
      // Instead, when an exception occurs, we return false and return a
      // description of the error in [describeMismatch] with the information
      // stored in [matchState].
      matchState[_internalErrorKey] = e;
      matchState[_internalStackTraceKey] = stackTrace;
      return false;
    }
  }

  static const _internalErrorKey = 'internal_error';
  static const _internalStackTraceKey = 'internal_error_stackTrace';

  @override
  Description describe(Description description) =>
      description.addDescriptionOf(description);

  @override
  Description describeMismatch(
    item,
    Description description,
    Map matchState,
    bool verbose,
  ) {
    // If an error was thrown in [matches] we print it from here
    if (matchState.containsKey(_internalErrorKey)) {
      // Add the error and stack trace to the description.
      final error = matchState[_internalErrorKey];
      final stackTrace = matchState[_internalStackTraceKey];
      return description
          .add('Failed:\n')
          .add('$error\n\n')
          .add('🔍 Detailed Stack trace:\n$stackTrace\n');
    }

    return _matcher.describeMismatch(
      item,
      description,
      matchState,
      verbose,
    );
  }
}

class _FixtureFileDoesNotExistsException implements Exception {
  final String file;

  _FixtureFileDoesNotExistsException({required this.file});

  @override
  String toString() => 'Fixture File does not exist: $file';
}
